from unittest import TestCase, TestSuite, main, makeSuite
import snippetfind

class QueryTestCase(TestCase):
    
    def test_multiple(self):
        self.assertEquals(
            [3, 1, 1],
            snippetfind.Query('search light').id_to_score)

    def test_simple(self):
        self.assertEquals(
            [3],
            snippetfind.Query('elephant').id_to_score)

# should really never happen, have assert in code..
##    def test_createPhraseEmpty(self):    
##         self.assertEquals(
##             {},
##             snippetfind.Phrase('').score_map)

##     def test_createPhraseMapWhitespace(self):
##         self.assertEquals(
##             {},
##             snippetfind.createPhraseMap('  '))

        
class TextMapTestCase(TestCase):
    def test_simple(self):
        m = snippetfind.TextMap(
            snippetfind.Query('elephant'), 'the elephant is fun')
        self.assertEquals(
            {4: 3},
            m.index_to_score)
        
    def test_twoTimes(self):
        m = snippetfind.TextMap(
            snippetfind.Query('elephant'), 'the elephant is an elephant')
        
        self.assertEquals(
            {4: 3, 19: 3},
            m.index_to_score)

    def test_multipleWords(self):
        m = snippetfind.TextMap(
            snippetfind.Query('elephant cow'), 'the elephant is a cow')
        self.assertEquals(
            {4: 1, 18: 1},
            m.index_to_score)

    def test_multipleWordsReverse(self):
        m = snippetfind.TextMap(
            snippetfind.Query('cow elephant'), 'the elephant is a cow')
        self.assertEquals(
            {4: 1, 18: 1},
            m.index_to_score)
    
    def test_case(self):
        m = snippetfind.TextMap(
            snippetfind.Query('elephant'), 'the elEPHANT is a cow')
        
        self.assertEquals(
            {4:3},
            m.index_to_score)

    def test_notfindmiddle(self):
        m = snippetfind.TextMap(
            snippetfind.Query('elephant'), 'theelephantishere')
        self.assertEquals(
            {},
            m.index_to_score)

class PartitionTestCase(TestCase):
    def test_partitionRoot(self):
        text = 'the car said to the lady, we are alone but heady.'
        root = snippetfind.RootPartition(snippetfind.Query('car'), text)
        self.assertEquals(
            0,
            root.getPosition())
        self.assertEquals(
            len(text),
            root.getSize())
        self.assertEquals(
            (0, len(text)),
            root.getRange())
        self.assertEquals(
            3,
            root.getScore())

    def test_partition2(self):
        text = 'the car said to the lady, we are alone but heady.'
        root = snippetfind.RootPartition(snippetfind.Query('car'), text)
        partitions = root.getPartitions(2)
        # 'rest partition' makes this up to 3 partitions
        self.assertEquals(1, len(partitions))
        self.assertEquals(
            len(text) / 2,
            partitions[0].getSize())
        self.assertEquals(
            0,
            partitions[0].getPosition())
        self.assertEquals(
            3,
            partitions[0].getScore())
        
    def test_partition3(self):
        text = 'the car said to the lady, you are a car yourself.'
        root = snippetfind.RootPartition(snippetfind.Query('car'), text)
        partitions = root.getPartitions(2)
        partitions.sort(lambda x, y: cmp(x.getPosition(), y.getPosition()))    
        self.assertEquals(
            len(text) / 2,
            partitions[0].getSize())
        self.assertEquals(
            len(text) / 2,
            partitions[1].getSize())
        self.assertEquals(
            0,
            partitions[0].getPosition())
        self.assertEquals(
            partitions[0].getSize(),
            partitions[1].getPosition())
        self.assertEquals(
            3,
            partitions[0].getScore())
        self.assertEquals(
            3,
            partitions[1].getScore())

    def test_subpartition(self):
        text = 'the car said to the lady, I am a car, but you are a lady'
        root = snippetfind.RootPartition(snippetfind.Query('car'), text)
        partition = root.getPartition(8, len(text) - 8)
        self.assertEquals(3, partition.getScore())
        
    def test_partitionOffset(self):
        text = 'a a b b a a'
        root = snippetfind.RootPartition(snippetfind.Query('b'), text)
        partitions = root.getTopScorePartitionsUseOffset(3, 1)
        for partition in partitions:
            start, end = partition.getRange()
            #print text[start:end], partition.getScore()
        #root = snippetfind.RootPartition(text, 'car')
        #partitions = root.getPartitions(3)
        #t = []
        #for partition in partitions:
        #    start, end = partition.getRange()
        #    t.append(text[start:end] + str(partition.getScore()))
        
        #print '|'.join(t)
        #] for in partitions
        #partitions = root.getTopScorePartitionsUseOffset(3, 1)
        #start, end = partitions[0].getRange()
        #print text[start:end], partitions[0].getScore()
        
        #partitions = root.getPartitions(3, len(text) / 6)
        #for partition in partitions:
        #    print partition.getScore()

    def test_partitionOffsetEmptyText(self):
        text = ''
        root = snippetfind.RootPartition(snippetfind.Query('b'), text)
        partitions = root.getTopScorePartitionsUseOffset(3, 1)
        for partition in partitions:
            start, end = partition.getRange()
        
    def test_top1(self):
        text = 'a a b b b a a a'
        #       012345678901234
        #       0         1
        root = snippetfind.RootPartition(snippetfind.Query('b'), text)
        partitions = root.getTopScorePartitionsNoOverlap(2, 1)
        self.assertEquals((4, 9),
                          partitions[0].getExtendedRange())
        
    def test_top2(self):
        text = 'a a a b b a a a a b b b a a a a b b b b a a a a b b b'
        #       01234567890123456789012345678901234567890123456789012
        #       0         1         2         3         4         5
        root = snippetfind.RootPartition(snippetfind.Query('b'), text)
        partitions = root.getTopScorePartitionsNoOverlap(6, 1)
        self.assertEquals((32, 41), partitions[0].getExtendedRange())

    def test_top3(self):
        text = 'a a a a b b b b'
        root = snippetfind.RootPartition(snippetfind.Query('c'), text)
        partitions = root.getTopScorePartitionsNoOverlap(6, 1)
        self.assertEquals(0, len(partitions))

    def test_top4(self):
        text = 'a a a b a a a c a a a b'
        #       01234567890123456789012
        root = snippetfind.RootPartition(snippetfind.Query('b c'), text)
        partitions = root.getTopScorePartitionsNoOverlap(6, 3)
        self.assertEquals(3, len(partitions))

    def test_top5(self):
        text = 'a a a b c a a a c a a a b'
        root = snippetfind.RootPartition(snippetfind.Query('b c'), text)
        partitions = root.getTopScorePartitionsNoOverlap(6, 3)
        self.assertEquals(3, len(partitions))
           
    def test_partition4(self):
        text = '0123456'
        root = snippetfind.RootPartition(snippetfind.Query('0'), text)
##         partitions = root.getPartitions(2)
##         self.assertEquals(
##             len(text) / 2,
##             partitions[0].getSize())
##         self.assertEquals(
##             len(text) / 2,
##             partitions[1].getSize())
##         self.assertEquals(
##             1,
##             partitions[2].getSize())
##         self.assertEquals(
##             6,
##             partitions[2].getPosition())

    def test_extendedrange1(self):
        text = 'alpha beta gamma delta foo bar baz qux'
        #       01234567890123456789012345678901234567
        #       0         1         2         3
        root = snippetfind.RootPartition(snippetfind.Query('gamma'), text)
        p = root.getPartition(11, 10)
        self.assertEquals((11, 22), p.getExtendedRange(4))

    def test_extendedrange2(self):
        text = 'alpha beta gamma delta foo bar baz qux'
        #       01234567890123456789012345678901234567
        #       0         1         2         3
        root = snippetfind.RootPartition(snippetfind.Query('gamma'), text)
        p = root.getPartition(7, 10)
        self.assertEquals((11, 22), p.getExtendedRange(10))

    def test_extendedrange3(self):
        text = 'alpha beta gamma delta foo bar baz qux'
        #       01234567890123456789012345678901234567
        #       0         1         2         3
        root = snippetfind.RootPartition(snippetfind.Query('gamma'), text)
        p = root.getPartition(10, 10)
        self.assertEquals((11, 22), p.getExtendedRange(10))

    def test_extendedrange4(self):
        text = 'alpha beta gamma delta foo bar baz qux'
        #       01234567890123456789012345678901234567
        #       0         1         2         3
        root = snippetfind.RootPartition(snippetfind.Query('gamma'), text)
        p = root.getPartition(10, 3)
        self.assertEquals((11, 16), p.getExtendedRange(10))

    def test_extendedrange5(self):
        text = 'alpha beta gamma delta foo bar baz qux'
        #       01234567890123456789012345678901234567
        #       0         1         2         3
        root = snippetfind.RootPartition(snippetfind.Query('gamma'), text)
        p = root.getPartition(2, 12)
        self.assertEquals((6, 16), p.getExtendedRange(10))

    def test_find_text(self):
        text = 'test paragraph'
        root = snippetfind.RootPartition(snippetfind.Query('paragraph'), text)
        partitions = root.getTop(80, 3)
        self.assertEquals(1, len(partitions))
        self.assertEquals('test paragraph', partitions[0].getText())
        
        
def test_suite():
    return TestSuite((
        makeSuite(QueryTestCase),
        makeSuite(TextMapTestCase),
        makeSuite(PartitionTestCase)))



if __name__=='__main__':
    main(defaultTest='test_suite')
